开源硬件起步的 8 个步骤
============================================================

> 制造自己的硬件比以往任何时候都更容易，更便宜。以下是你设计、构建和测试你的第一块板子所需的事情。


![8 ways to get started with open source hardware](https://opensource.com/sites/default/files/styles/image-full-size/public/images/life/hardware_hammer_sign.jpg?itok=vh76LBib "8 ways to get started with open source hardware")

著名的计算机科学家阿伦凯（Alan Kay）曾经说过：“认真对待软件的人应该制造他们自己的硬件。” 我认为如今就如 同 1982 年他所说的一样。然而，现在和那时之间的不同是硬件变得越来越快、越来越小，最重要的是：更便宜。 现在可以用 5 美元购买一台完整的电脑。

随着大公司降低自己产品的价格，能够生产生产级硬件的制造业生态系统得以增长，这些硬件的成本足够便宜，并且达到了普通人都可以接受的程度。这种可用性以及可负担性正在帮助推动诸如众筹和创客运动之类的事情，但同时它们也让更多的个人能够通过开源硬件参与到开源当中。

> 探索开源硬件
> *   [什么是开源硬件？][1]
> *   [什么是树莓派？][2] 
> *   [什么是 Arduino？][3]

开源硬件和非开源硬件有很多区别，但是开源硬件联盟（OSHWA）定义了一个大多数人同意的定义，如果你熟悉开源软件，这不会听上去太奇怪：

> “开源硬件（OSHW）是一个指有形的造物：机器、设备或者其它物理东西的术语——其设计向公众发布，任何人可以制造、修改、分发并使用那些造物。”

我们身边已经有很多开源硬件了。你可能没有注意到你在使用的主板实际上可能是开源硬件。从低调而多能的 [Arduino][13]，一直到像 [BeagleBone][14] 系列和 [C.H.I.P.][15] 计算机这样的完整功能的电脑，有很多开源硬件的例子，还有更多的在设计中。

硬件可能很复杂，对初学者而言有时候不太理解为什么设计需要某些东西。但开源硬件使你不仅可以看到工作示例，还可以更改这些设计，或者在你自己的设计中剔除或复制所需的部分，就如复制和粘贴一样简单。

### 我该如何开始？

我们先要指出硬件很“硬”，它很复杂甚至很深奥，你可能用到的工具并不总是最人性化的。任何一个玩微处理器的时间足够长的人都可以向你证实：你会烧坏一些东西，看到神奇的烟雾在某个时刻冒出来。没关系，我们都遇到过，有些人还会反复遭遇，因为我们在做一件事情的前 100 次时都不会得到教训，但不要让这些阻碍你：当做错事情时，你会学到教训，而且你将来还可以将有趣的故事告诉别人。

#### 建模

首先要做的是使用现有的电路板、跳线、面包板以及你要连接的任何设备来建模你想要做的事情。在许多情况下，最简单的事情就是在板上添加更多的 LED，并以新颖的方式让它们闪烁起来。这是一个很好的做出原型的方式，也是一个常见的做法。它看上去并不漂亮，你可能会发现你的线接错了，但这些都是原型 - 你只是想证明硬件可以工作。当硬件不工作时，一定要仔细检查一切，不要害怕寻求帮助 - 有时第二双眼睛会发现你奇怪的接地短路。

#### 设计

当你弄清楚你想要构建的硬件，现在是时候把你的想法从跳线和面包板变成实际的设计了。这时事情会变得让人气馁，但是从小处开始，事情上，可以从熟悉加工和处理这样非常小的地方开始，所以为什么不从制作一块带有 LED 和电池的印刷电路板开始？认真地说，这可能听起来过于简单，但在这里有很多新的基础要了解。

1. **找到一个电子设计自动化（EDA）工具来使用。** 有很多好的开源软件可以选择，但是它们并不总是用户友好的。[Fritzing][5]、[gEDA][6] 还有 [KiCad][7] 都是开源的，并且其可用性一个比一个好。如果你想要尝试更多的商业软件，那么还有一些其他的选择。Eagle 有个受限的免费版本可供使用，有许多的开源硬件是用它设计的。

2. **在 EDA 工具中设计你的电路板。** 依据你选择的工具，这可能会非常快，或者可能是学习如何设计的很好的练习。这是我建议从小的硬件开始的原因之一。一个带 LED 的电路可以如一块电池、一个电阻、一个 LED 一样简单。电路图非常简单，并且板子也会非常小、非常简单。

3. **为打样而导出设计。** 这与列表中的下一件事情紧密相连，但如果你以前没有这样做过，这也可能是一个令人困惑的过程。当你在导出时，你会有很多细节需要调整，并且需要以某种方式导出以便电路板工厂能确切知道你要做的。

4. **找到一个电路板工厂。** 有许多电路板工厂可以制作你的设计，并且一些比其他更加友好及有帮助。一个特别棒的地方是 [OSH Park][8]，这些人非常友好并支持开源硬件。他们也有一个非常扎实的流程来确认你发送给它的就是会被制造的，所以他们值得一试。还有很多其他选择；看看 [PCB Shopper][9]，它可以让你比较不同实体 PCB 商家的价格、周转时间等等。

5. **等待。** 这或许是在制造你自己的电路板中最难的一部分了，因为它会花费时间将数字部分变成物理产品。计划好两周时间来拿到你的电路板。这是你继续下个项目的绝好时间，买到或确保你当前制造的所有部分都有了，或者随便试试而不要担心。你的第一块电路板是艰难的 - 你现在非常想要，但是保持耐心。

6. **修补并提升。** 一旦拿到你的板子，是时候上电测试了。如果你是以 LED 电路开始，那么它很容易调试，并且你会得到一些可以工作起来的东西。如果你有更复杂的电路，那么需要有条理并且有耐心。有时候电路不工作，并且你需要用你的调试技能来追踪问题。

7. **最后，如果你做的是开源硬件，那就发布它。*** 我们谈论的是开源硬件，因此确保它包含了一个许可，发布它、共享它，把它放在人们可以看见你所做的地方。你或许会想写一篇博客并提交到如 Hackaday 上面。

8. **最重要的是，玩得开心。** 坦白说，如果你在做一些事但是你不开心，你应该停止这样做。开源硬件可以很有趣，虽然有时是困难而且复杂的。但是不是一切都工作：见鬼，我已经设计了一半的电路不工作；或者我（意外地）在电源和接地之间造成了 12 次短路；这些电路板是双层板：是的。我在这个过程中学到了一些东西：非常多，并且我不会再犯同样的错误。我会做出新的板子，但不是这些。（我会支持并盯着这些板子和它们的错误，悲伤的是，它们不会在我盯着它们时感到良心会痛）。

现在有许多的开源硬件，有许多好的例子从中可以查看、复制、衍生，并且有很多信息使制造硬件变得简单。这就是开源硬件：一个人们制造它们、共享它们的社区，每个人可以制作他们自己的东西并构建他们想要的硬件——而不是他们可以得到的硬件。

（题图：Thomas Hawk on [Flickr][11]. [CC BY-NC 2.0][12]. 由 Opensource.com 修改）

--------------------------------------------------------------------------------



作者简介：

John 'Warthog9' Hawley - John 在 VMWare 的开源项目办公室为上游开源项目工作。在以前的生活中，他曾在 MinnowBoard 开源硬件项目上工作，领导了 kernel.org 的系统管理团队，并在桌面集群变得很酷之前构建了它们。为了乐趣，他构建了多个明星项目，比如一个受欢迎的英国电视节目 K-9 的复制品，在无人机的飞行计算机视觉处理中完成，设计并制作了一堆自己的硬件。

-------------------------------------

via: https://opensource.com/article/17/5/8-ways-get-started-open-source-hardware

作者：[John 'Warthog9' Hawley][a]
译者：[geekpi](https://github.com/geekpi)
校对：[wxy](https://github.com/wxy)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://opensource.com/article/17/5/8-ways-get-started-open-source-hardware
[1]:https://opensource.com/resources/what-open-hardware?src=open_hardware_resources_menu
[2]:https://opensource.com/resources/what-raspberry-pi?src=open_hardware_resources_menu
[3]:https://opensource.com/resources/what-arduino?src=open_hardware_resources_menu
[4]:https://opensource.com/tags/hardware?src=open_hardware_resources_menu
[5]:http://fritzing.org/home/
[6]:http://www.geda-project.org/
[7]:http://kicad-pcb.org/
[8]:https://oshpark.com/
[9]:http://pcbshopper.com/
[10]:https://opensource.com/article/17/5/8-ways-get-started-open-source-hardware?rate=jPBGDIa2vBXW6kb837X8JWdjI2V47hZ4KecI8-GJBjQ
[11]:https://www.flickr.com/photos/thomashawk/3048157616/in/photolist-5DmB4E-BzrZ4-5aUXCN-nvBWYa-qbkwAq-fEFeDm-fuZxgC-dufA8D-oi8Npd-b6FiBp-7ChGA3-aSn7xK-7NXMyh-a9bQQr-5NG9W7-agCY7E-4QD9zm-7HLTtj-4uCiHy-bYUUtG
[12]:https://creativecommons.org/licenses/by-nc/2.0/
[13]:https://opensource.com/node/20751
[14]:https://opensource.com/node/35211
[15]:https://opensource.com/node/24891
[16]:https://opensource.com/user/130046/feed
[17]:https://opensource.com/users/warthog9
